#!/bin/sh

. "$_EXEC/indexmeta.sh"
. "$_EXEC/widgets.sh"

list_item() {
  local name link path length width height tags comment n
  name="$(HTML "$1")"
  link="$(URL "$ITEM/$1")"
  path="$(HTML "$ITEM/$1")"
  qry=$(HTML "$QUERY_STRING")

  if [ -d "$_DATA/$ITEM/$1" ]; then
    printf '[a .list .dir href="%s" %s]' "${path}?${qry}" "$name"
  elif [ -f "$_DATA/$ITEM/$1" ]; then
    read -r length width height tags comment n <<-EOF
	$(meta_info "$_DATA/$ITEM/$1")
	EOF
    printf '[div .list .file
              [a href="%s" [img src="%s?a=thumbnail"]][label %s]
              [span .time %i:%imin] [span .dim %ix%i] %s
              [checkbox "select" "%s" id="select_%s"][label for="select_%s" +]
            ]' \
      "$link" "$link" "$name" \
      "$((length / 60))" "$((length % 60))" \
      "$width" "$height" \
      "$(UNSTRING "${tags#tags=}" |tr , '\0' |xargs -r0 printf ' [span .tag %s]')" \
      "$path" "$path" "$path"
  else
    printf 'Canning record for nonexist file: %s\n' "$1" >&2
    meta_purge "$_DATA/$ITEM/$1"
  fi
}

list_directories(){
  (cd "$_DATA/$ITEM";
    find ./ -type d \! -name .index -mindepth 1 -maxdepth 1 \
  ) | cut -d/ -f2- | sort
}

[ "$FILTER" ] && list_fex="$(
  fex='p'
  STRING "$FILTER^" \
  | sed -r 's;\^;\n;g; s;[]\/\(\)\\\^\$\?\.\+\*\;\[\{\}];\\&;g' \
  | while read -r f; do
    [ ! "${f#~}" ] && continue
    [ "${f#~}" = "$f" ] \
    && fex="/(\ttags=([^\t]*,)?)(${f})((,[^\t]*)?\t)/{${fex}}" \
    || fex="/(\ttags=([^\t]*,)?)(${f#~})((,[^\t]*)?\t)/d; ${fex}"
    printf '%s\n' "${fex}"
  done \
  | tail -n1
)"

list_fullname(){
  sn="$1"
  [ ! "${sn%%*/*}" ] && base="${sn%/*}" || base=.
  file="$(printf '%s' "$_DATA/$ITEM/$sn".*)"
  file="${file##*/}"
  [ -e "$_DATA/$ITEM/$base/${file}" ] \
  && printf '%s\n' "${base}/${file}"
}

list_filemeta(){
  local meta base f fn file
  base="$1"
  meta="$_DATA/$ITEM/$base/.index/meta"
  meta_dir "$_DATA/$ITEM/$base"

  if [ $ORDER = Name ]; then
    sort -k6 "$meta"
  elif [ $ORDER = Length ]; then
    sort -n -k1 "$meta"
  else
    cat "$meta"
  fi \
  | if [ "$FILTER" ]; then
    sed -nr "$list_fex"
  elif [ "${SEARCH#!}" != "${SEARCH}" ]; then
    grep -aviE "$(STRING "${SEARCH#!}" |sed -r ':x s;((^|[^\\])(\\\\)*)\+;\1\\+;g; tx;')"
  else
    grep -aiE "$(STRING "${SEARCH}" \
                 | sed -r ':x s;((^|[^\\])(\\\\)*)\+;\1 ;g; tx;
                            s;((^|[^\\])(\\\\)*)\\\+;\1+;g;
                            s; ;\\+;g;')"
  fi |cut -f6 | UNSTRING \
  | while read -r fn; do
    printf '%s\n' "${base}/${fn%${CR}}"
  done | tee .index/debug
}

list_index(){
  local meta
  (cd "$_DATA/$ITEM";
    find ./ -path '*/.index/meta'
  ) | while read -r meta; do
    list_filemeta "${meta%/.index/meta}"
  done
}

list_dateorder(){
  while read -r sn; do 
    list_fullname "$sn"
  done \
  | xargs -rd'\n' stat -c '%Y	%n' \
  | sort -rn |cut -d/ -f2- \
  | sed -r 's;\.[^\.]*$;;;'
}

list_items() {
  local mode
  mode="$(COOKIE mode |grep -m1 -axE 'index|browse' || printf browse )"

  [ "$mode" = browse -a "$ITEM" ] && printf '..\n'

  if   [ "$mode" = browse -a "$ORDER" = Date ]; then
    list_directories
    list_filemeta . |list_dateorder
  elif [ "$mode" = index  -a "$ORDER" = Date ]; then
    list_index |list_dateorder
  elif [ "$mode" = browse ]; then
    list_directories
    list_filemeta .
  elif [ "$mode" = index ]; then
    list_index
  fi
}

list_paginate() {
  local page i c n end
  page="$(GET p |grep -axE '[0-9]+' || printf 1)"; c=1
  end=$((page + LISTSIZE))

  printf '[div .itemlist '
  while read -r i; do
    [ $c -ge $page -a $c -lt $end  ] \
    && list_item "$(list_fullname "$i")"
    c=$((c + 1))
  done
  printf ']'

  printf '[div .pagination'
  for n in $( seq 1 $((c / LISTSIZE + 1)) ); do
    printf '[a .page href="%s" %s]' \
      "?p=$(( (n - 1) * LISTSIZE + 1))&$qry" "$n"
  done
  printf ']'
}

printf 'Content-Type: text/html;charset=utf-8\r\n\r\n'

{ printf '
[!DOCTYPE HTML]
[html [head [title Listing]
  [meta name="viewport" content="width=device-width"]
  [link rel=stylesheet href="/style.css" ]
] [body
  [div #navigation
    [a #t_bookmarks href="#bookmarks" &#x2605;]'
    w_search
    printf '
    [a #t_avsearch href="#advsearch" Advanced]
    [a #t_prefs href="#prefs" &#x2699;]
  ]'
  w_bookmarks
  w_advsearch
  w_prefs
  printf '
  [form method=POST action="?a=multitag"'
    list_items \
    | list_paginate
    printf '
    [div #editing
      [a href="#multitag" Add Tags]'
      w_tagging
      w_index
      printf '
    ]
  ]
] ]
'; } | "$_EXEC/cgilite/html-sh.sed"
